# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.kumulos import NotifyKumulos

logging.disable(logging.CRITICAL)

# a test UUID we can use
UUID4 = "8b799edf-6f98-4d3a-9be7-2862fb4e5752"

# Our Testing URLs
apprise_url_tests = (
    (
        "kumulos://",
        {
            # No API or Server Key specified
            "instance": TypeError,
        },
    ),
    (
        "kumulos://:@/",
        {
            # No API or Server Key specified
            # We don't have strict host checking on for kumulos, so this URL
            # actually becomes parseable and :@ becomes a hostname.
            # The below errors because a second token wasn't found
            "instance": TypeError,
        },
    ),
    (
        f"kumulos://{UUID4}/",
        {
            # No server key was specified
            "instance": TypeError,
        },
    ),
    (
        "kumulos://{}/{}/".format(UUID4, "w" * 36),
        {
            # Everything is okay
            "instance": NotifyKumulos,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "kumulos://8...2/w...w/",
        },
    ),
    (
        "kumulos://{}/{}/".format(UUID4, "x" * 36),
        {
            "instance": NotifyKumulos,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "kumulos://8...2/x...x/",
        },
    ),
    (
        "kumulos://{}/{}/".format(UUID4, "y" * 36),
        {
            "instance": NotifyKumulos,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "kumulos://8...2/y...y/",
        },
    ),
    (
        "kumulos://{}/{}/".format(UUID4, "z" * 36),
        {
            "instance": NotifyKumulos,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_kumulos_urls():
    """NotifyKumulos() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


def test_plugin_kumulos_edge_cases():
    """NotifyKumulos() Edge Cases."""

    # Invalid API Key
    with pytest.raises(TypeError):
        NotifyKumulos(None, None)
    with pytest.raises(TypeError):
        NotifyKumulos("     ", None)

    # Invalid Server Key
    with pytest.raises(TypeError):
        NotifyKumulos("abcd", None)
    with pytest.raises(TypeError):
        NotifyKumulos("abcd", "       ")



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""IRC templates used to auto-apply defaults based on known networks.

This mirrors the email template strategy, but is intentionally minimal.
Templates only apply defaults when the user has not explicitly specified the
value in their URL.

Each entry is a tuple of:
  (label, compiled_regex, defaults)

Supported defaults:
  - secure: bool
  - port: int
  - mode: str  (server|nickserv|none)
"""

from __future__ import annotations

import re

from .protocol import IRCAuthMode

IRC_TEMPLATES = (
    # SynIRC: NickServ common, TLS default 6697
    (
        "SynIRC",
        re.compile(r"^.+\.synirc\.net$", re.I),
        {
            "secure": True,
            "port": 6697,
            "mode": IRCAuthMode.NICKSERV,
        },
    ),
    # Libera.Chat: TLS default 6697, NickServ common (SASL also common but
    # not implemented here)
    (
        "Libera.Chat",
        re.compile(r"^.+\.libera\.chat$", re.I),
        {
            "secure": True,
            "port": 6697,
            "mode": IRCAuthMode.NICKSERV,
        },
    ),
    # EFnet: traditionally plain 6667, auth varies widely
    (
        "EFnet",
        re.compile(r"^.+\.efnet\.org$", re.I),
        {
            "secure": False,
            "port": 6667,
            "mode": IRCAuthMode.SERVER,
        },
    ),
)



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
from json import loads
import logging
import os
from unittest import mock

from helpers import AppriseURLTester
import requests

from apprise import Apprise, AppriseAttachment, NotifyType
from apprise.plugins.apprise_api import NotifyAppriseAPI

logging.disable(logging.CRITICAL)

# Attachment Directory
TEST_VAR_DIR = os.path.join(os.path.dirname(__file__), "var")

# Our Testing URLs
apprise_url_tests = (
    (
        "apprise://",
        {
            # invalid url (not complete)
            "instance": None,
        },
    ),
    # A a bad url
    (
        "apprise://:@/",
        {
            "instance": None,
        },
    ),
    # No token specified
    (
        "apprise://localhost",
        {
            "instance": TypeError,
        },
    ),
    # invalid token
    (
        "apprise://localhost/!",
        {
            "instance": TypeError,
        },
    ),
    # No token specified (whitespace is trimmed)
    (
        "apprise://localhost/%%20",
        {
            "instance": TypeError,
        },
    ),
    # A valid URL with Token
    (
        "apprise://localhost/%s" % ("a" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprise://localhost/a...a/",
            "force_debug": True,
        },
    ),
    # A valid URL with long Token
    (
        "apprise://localhost/%s" % ("a" * 128),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprise://localhost/a...a/",
        },
    ),
    # A valid URL with Token (using port)
    (
        "apprise://localhost:8080/%s" % ("b" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprise://localhost:8080/b...b/",
        },
    ),
    # A secure (https://) reference
    (
        "apprises://localhost/%s" % ("c" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://localhost/c...c/",
        },
    ),
    # Native URL suport (https)
    (
        "https://example.com/path/notify/%s" % ("d" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://example.com/path/d...d/",
        },
    ),
    # Native URL suport (http)
    (
        "http://example.com/notify/%s" % ("d" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprise://example.com/d...d/",
        },
    ),
    # support to= keyword
    (
        "apprises://localhost/?to=%s" % ("e" * 32),
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprises://localhost/e...e/",
        },
    ),
    # support token= keyword (even when passed with to=, token over-rides)
    (
        "apprise://localhost/?token={}&to={}".format("f" * 32, "abcd"),
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprise://localhost/f...f/",
        },
    ),
    # Test tags
    (
        "apprise://localhost/?token={}&tags=admin,team".format("abcd"),
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprise://localhost/a...d/",
        },
    ),
    # Test Format string
    (
        "apprise://user@localhost/mytoken0/?format=markdown",
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprise://user@localhost/m...0/",
        },
    ),
    (
        "apprise://user@localhost/mytoken1/",
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprise://user@localhost/m...1/",
        },
    ),
    (
        "apprise://localhost:8080/mytoken/",
        {
            "instance": NotifyAppriseAPI,
        },
    ),
    (
        "apprise://user:pass@localhost:8080/mytoken2/",
        {
            "instance": NotifyAppriseAPI,
            "privacy_url": "apprise://user:****@localhost:8080/m...2/",
        },
    ),
    (
        "apprises://localhost/mytoken/",
        {
            "instance": NotifyAppriseAPI,
        },
    ),
    (
        "apprises://user:pass@localhost/mytoken3/",
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://user:****@localhost/m...3/",
        },
    ),
    (
        "apprises://localhost:8080/mytoken4/",
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://localhost:8080/m...4/",
        },
    ),
    (
        "apprises://localhost:8080/abc123/?method=json",
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://localhost:8080/a...3/",
        },
    ),
    (
        "apprises://localhost:8080/abc123/?method=form",
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://localhost:8080/a...3/",
        },
    ),
    # Invalid method specified
    (
        "apprises://localhost:8080/abc123/?method=invalid",
        {
            "instance": TypeError,
        },
    ),
    (
        "apprises://user:password@localhost:8080/mytoken5/",
        {
            "instance": NotifyAppriseAPI,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "apprises://user:****@localhost:8080/m...5/",
        },
    ),
    (
        "apprises://localhost:8080/path?+HeaderKey=HeaderValue",
        {
            "instance": NotifyAppriseAPI,
        },
    ),
    (
        "apprise://localhost/%s" % ("a" * 32),
        {
            "instance": NotifyAppriseAPI,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "apprise://localhost/%s" % ("a" * 32),
        {
            "instance": NotifyAppriseAPI,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "apprise://localhost/%s" % ("a" * 32),
        {
            "instance": NotifyAppriseAPI,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_apprise_api_urls():
    """NotifyAppriseAPI() General Checks."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_notify_apprise_api_payload_check(mock_post):
    """NotifyAppriseAPI() payload checks"""

    okay_response = requests.Request()

    okay_response.status_code = requests.codes.ok
    okay_response.content = ""

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    obj = Apprise.instantiate("apprise://user@localhost/mytoken1/?method=form")
    assert isinstance(obj, NotifyAppriseAPI)

    # Test URL with Attachment
    path = os.path.join(TEST_VAR_DIR, "apprise-test.gif")
    attach = AppriseAttachment(path)
    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=attach,
        )
        is True
    )

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://localhost/notify/mytoken1"
    assert details[1]["data"] == {
        "title": "title",
        "body": "body",
        "type": "info",
        "format": "text",
    }
    assert "X-Apprise-ID" in details[1]["headers"]
    assert details[1]["headers"].get("User-Agent") == "Apprise"
    assert details[1]["headers"].get("Accept") == "application/json"
    assert details[1]["headers"].get("X-Apprise-Recursion-Count") == "1"

    mock_post.reset_mock()

    obj = Apprise.instantiate("apprise://user@localhost/mytoken1/?method=json")
    assert isinstance(obj, NotifyAppriseAPI)

    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=attach,
        )
        is True
    )

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://localhost/notify/mytoken1"
    data = loads(details[1]["data"])
    assert "attachments" in data
    assert isinstance(data["attachments"], list)
    assert len(data["attachments"]) == 1

    # Remove our attachment to make the next assert easier
    del data["attachments"]
    assert data == {
        "title": "title",
        "body": "body",
        "type": "info",
        "format": "text",
    }
    assert "X-Apprise-ID" in details[1]["headers"]
    assert details[1]["headers"].get("User-Agent") == "Apprise"
    assert details[1]["headers"].get("Accept") == "application/json"
    assert details[1]["headers"].get("X-Apprise-Recursion-Count") == "1"


@mock.patch("requests.post")
def test_notify_apprise_api_attachments(mock_post):
    """NotifyAppriseAPI() Attachments."""

    okay_response = requests.Request()

    for method in ("json", "form"):
        okay_response.status_code = requests.codes.ok
        okay_response.content = ""

        # Assign our mock object our return value
        mock_post.return_value = okay_response

        obj = Apprise.instantiate(
            f"apprise://user@localhost/mytoken1/?method={method}"
        )
        assert isinstance(obj, NotifyAppriseAPI)

        # Test Valid Attachment
        path = os.path.join(TEST_VAR_DIR, "apprise-test.gif")
        attach = AppriseAttachment(path)
        assert (
            obj.notify(
                body="body",
                title="title",
                notify_type=NotifyType.INFO,
                attach=attach,
            )
            is True
        )

        # Test invalid attachment
        path = os.path.join(
            TEST_VAR_DIR, "/invalid/path/to/an/invalid/file.jpg"
        )
        assert (
            obj.notify(
                body="body",
                title="title",
                notify_type=NotifyType.INFO,
                attach=path,
            )
            is False
        )

        # Test Valid Attachment (load 3)
        path = (
            os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
            os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
            os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
        )
        attach = AppriseAttachment(path)

        # Return our good configuration
        mock_post.side_effect = None
        mock_post.return_value = okay_response
        with mock.patch("builtins.open", side_effect=OSError()):
            # We can't send the message we can't open the attachment for
            # reading
            assert (
                obj.notify(
                    body="body",
                    title="title",
                    notify_type=NotifyType.INFO,
                    attach=attach,
                )
                is False
            )

        with mock.patch("requests.post", side_effect=OSError()):
            # Attachment issue
            assert (
                obj.notify(
                    body="body",
                    title="title",
                    notify_type=NotifyType.INFO,
                    attach=attach,
                )
                is False
            )

        # test the handling of our batch modes
        obj = Apprise.instantiate("apprise://user@localhost/mytoken1/")
        assert isinstance(obj, NotifyAppriseAPI)

        # Now send an attachment normally without issues
        mock_post.reset_mock()

        assert (
            obj.notify(
                body="body",
                title="title",
                notify_type=NotifyType.INFO,
                attach=attach,
            )
            is True
        )
        assert mock_post.call_count == 1

        details = mock_post.call_args_list[0]
        assert details[0][0] == "http://localhost/notify/mytoken1"
        assert obj.url(privacy=False).startswith(
            "apprise://user@localhost/mytoken1/"
        )

        mock_post.reset_mock()



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise import NotifyType
from apprise.plugins.rocketchat import NotifyRocketChat

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "rocket://",
        {
            "instance": None,
        },
    ),
    (
        "rockets://",
        {
            "instance": None,
        },
    ),
    (
        "rocket://:@/",
        {
            "instance": None,
        },
    ),
    # No username or pass
    (
        "rocket://localhost",
        {
            "instance": TypeError,
        },
    ),
    # No room or channel
    (
        "rocket://user:pass@localhost",
        {
            "instance": TypeError,
        },
    ),
    # No valid rooms or channels
    (
        "rocket://user:pass@localhost/#/!/@",
        {
            "instance": TypeError,
        },
    ),
    # No user/pass combo
    (
        "rocket://user@localhost/room/",
        {
            "instance": TypeError,
        },
    ),
    # No user/pass combo
    (
        "rocket://localhost/room/",
        {
            "instance": TypeError,
        },
    ),
    # A room and port identifier
    (
        "rocket://user:pass@localhost:8080/room/",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rocket://user:****@localhost",
        },
    ),
    # A channel (using the to=)
    (
        "rockets://user:pass@localhost?to=#channel",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    # A channel
    (
        "rockets://user:pass@localhost/#channel",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    # A channel using token based
    (
        "rockets://user:token@localhost/#channel?mode=token",
        {
            "instance": NotifyRocketChat,
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    # Token is detected based o it's length
    (
        "rockets://user:{}@localhost/#channel".format("t" * 40),
        {
            "instance": NotifyRocketChat,
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    # Several channels
    (
        "rocket://user:pass@localhost/#channel1/#channel2/?avatar=Yes",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rocket://user:****@localhost",
        },
    ),
    # Several Rooms
    (
        "rocket://user:pass@localhost/room1/room2",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rocket://user:****@localhost",
        },
    ),
    # A room and channel
    (
        "rocket://user:pass@localhost/room/#channel?mode=basic&avatar=Yes",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "rocket://user:****@localhost",
        },
    ),
    # A user/pass where the pass matches a webtoken
    # to ensure we get the right mode, we enforce basic mode
    # so that web/token gets interpreted as a password
    (
        "rockets://user:pass%2Fwithslash@localhost/#channel/?mode=basic",
        {
            "instance": NotifyRocketChat,
            # The response text is expected to be the following on a success
            "requests_response_text": {
                "status": "success",
                "data": {
                    "authToken": "abcd",
                    "userId": "user",
                },
            },
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    # A room and channel
    (
        "rockets://user:pass@localhost/rooma/#channela",
        {
            # The response text is expected to be the following on a success
            "requests_response_code": requests.codes.ok,
            "requests_response_text": {
                # return something other then a success message type
                "status": "failure",
            },
            # Exception is thrown in this case
            "instance": NotifyRocketChat,
            # Notifications will fail in this event
            "response": False,
        },
    ),
    # A web token
    (
        "rockets://web/token@localhost/@user/#channel/roomid",
        {
            "instance": NotifyRocketChat,
            "privacy_url": "rockets://****@localhost/#channel/roomid",
        },
    ),
    (
        "rockets://user:web/token@localhost/@user/?mode=webhook",
        {
            "instance": NotifyRocketChat,
            "privacy_url": "rockets://user:****@localhost",
        },
    ),
    (
        "rockets://user:web/token@localhost?to=@user2,#channel2",
        {
            "instance": NotifyRocketChat,
        },
    ),
    (
        "rockets://web/token@localhost/?avatar=No",
        {
            # a simple webhook token with default values
            "instance": NotifyRocketChat,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "rockets://****@localhost/",
        },
    ),
    (
        "rockets://localhost/@user/?mode=webhook&webhook=web/token",
        {
            "instance": NotifyRocketChat,
            "privacy_url": "rockets://****@localhost/@user",
        },
    ),
    (
        "rockets://user:web/token@localhost/@user/?mode=invalid",
        {
            # invalid mode
            "instance": TypeError,
        },
    ),
    (
        "rocket://user:pass@localhost:8081/room1/room2",
        {
            "instance": NotifyRocketChat,
            # force a failure using basic mode
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "rockets://user:web/token@localhost?to=@user3,#channel3",
        {
            "instance": NotifyRocketChat,
            # force a failure using webhook mode
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "rocket://user:pass@localhost:8082/#channel",
        {
            "instance": NotifyRocketChat,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "rocket://user:pass@localhost:8083/#chan1/#chan2/room",
        {
            "instance": NotifyRocketChat,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_rocket_chat_urls():
    """NotifyRocketChat() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.get")
@mock.patch("requests.post")
def test_plugin_rocket_chat_edge_cases(mock_post, mock_get):
    """NotifyRocketChat() Edge Cases."""

    # Chat ID
    recipients = "AbcD1245, @l2g, @lead2gold, #channel, #channel2"

    # Authentication
    user = "myuser"
    password = "mypass"

    # Prepare Mock
    mock_get.return_value = requests.Request()
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.ok
    mock_get.return_value.status_code = requests.codes.ok
    mock_post.return_value.content = ""
    mock_get.return_value.content = ""

    obj = NotifyRocketChat(user=user, password=password, targets=recipients)
    assert isinstance(obj, NotifyRocketChat) is True
    assert len(obj.channels) == 2
    assert len(obj.users) == 2
    assert len(obj.rooms) == 1

    # No Webhook specified
    with pytest.raises(TypeError):
        obj = NotifyRocketChat(webhook=None, mode="webhook")

    #
    # Logout
    #
    assert obj.logout() is True

    # Invalid JSON during Login
    mock_post.return_value.content = "{"
    mock_get.return_value.content = "}"
    assert obj.login() is False

    # Prepare Mock to fail
    mock_post.return_value.content = ""
    mock_get.return_value.content = ""
    mock_post.return_value.status_code = requests.codes.internal_server_error
    mock_get.return_value.status_code = requests.codes.internal_server_error

    #
    # Send Notification
    #
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is False
    )
    assert obj._send(payload="test", notify_type=NotifyType.INFO) is False

    #
    # Logout
    #
    assert obj.logout() is False

    # KeyError handling
    mock_post.return_value.status_code = 999
    mock_get.return_value.status_code = 999

    #
    # Send Notification
    #
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is False
    )
    assert obj._send(payload="test", notify_type=NotifyType.INFO) is False

    #
    # Logout
    #
    assert obj.logout() is False

    # Generate exceptions
    mock_get.side_effect = requests.ConnectionError(
        0, "requests.ConnectionError() not handled"
    )
    mock_post.side_effect = mock_get.side_effect

    #
    # Send Notification
    #
    assert obj._send(payload="test", notify_type=NotifyType.INFO) is False

    # Attempt the check again but fake a successful login
    obj.login = mock.Mock()
    obj.login.return_value = True
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is False
    )
    #
    # Logout
    #
    assert obj.logout() is False



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from os.path import dirname, join
import re
import time
from unittest import mock
import urllib

import pytest

from apprise import AppriseAttachment, exception
from apprise.attachment.base import AttachBase
from apprise.attachment.file import AttachFile
from apprise.common import ContentLocation

logging.disable(logging.CRITICAL)

TEST_VAR_DIR = join(dirname(__file__), "var")


def test_attach_file_parse_url():
    """
    API: AttachFile().parse_url()

    """

    # bad entry
    assert AttachFile.parse_url("garbage://") is None

    # no file path specified
    assert AttachFile.parse_url("file://") is None


def test_file_expiry(tmpdir):
    """
    API: AttachFile Expiry
    """
    path = join(TEST_VAR_DIR, "apprise-test.gif")
    image = tmpdir.mkdir("apprise_file").join("test.jpg")
    with open(path, "rb") as data:
        image.write(data)

    aa = AppriseAttachment.instantiate(str(image), cache=30)

    # Our file is now available
    assert aa.exists()

    # Our second call has the file already downloaded, but now compares
    # it's date against when we consider it to have expire.  We're well
    # under 30 seconds here (our set value), so this will succeed
    assert aa.exists()

    with mock.patch("time.time", return_value=time.time() + 31):
        # This will force a re-download as our cache will have
        # expired
        assert aa.exists()

    with mock.patch("time.time", side_effect=OSError):
        # We will throw an exception
        assert aa.exists()


def test_attach_mimetype():
    """
    API: AttachFile MimeType()

    """
    # Simple gif test
    path = join(TEST_VAR_DIR, "apprise-test.gif")
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    assert response.path == path
    assert response.name == "apprise-test.gif"
    assert response.mimetype == "image/gif"

    # Force mimetype
    response._mimetype = None
    response.detected_mimetype = None

    assert response.mimetype == "image/gif"

    response._mimetype = None
    response.detected_mimetype = None
    with mock.patch("mimetypes.guess_type", side_effect=TypeError):
        assert response.mimetype == "application/octet-stream"


def test_attach_file():
    """
    API: AttachFile()

    """
    # Simple gif test
    path = join(TEST_VAR_DIR, "apprise-test.gif")
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    assert response.path == path
    assert response.name == "apprise-test.gif"
    assert response.mimetype == "image/gif"
    # Download is successful and has already been called by now; below pulls
    # results from cache
    assert response.download()

    with mock.patch("os.path.isfile", side_effect=OSError):
        assert response.exists() is False

    with mock.patch("os.path.isfile", return_value=False):
        assert response.exists() is False

    # Test that our file exists
    assert response.exists() is True
    response.cache = True
    # Leverage always-cached flag
    assert response.exists() is True

    # On Windows, it is:
    #  `file://D%3A%5Ca%5Capprise%5Capprise%5Ctest%5Cvar%5Capprise-test.gif`
    path_in_url = urllib.parse.quote(path)
    assert response.url().startswith(f"file://{path_in_url}")

    # No mime-type and/or filename over-ride was specified, so it
    # won't show up in the generated URL
    assert re.search(r"[?&]mime=", response.url()) is None
    assert re.search(r"[?&]name=", response.url()) is None

    # Test case where location is simply set to INACCESSIBLE
    # Below is a bad example, but it proves the section of code properly works.
    # Ideally a server admin may wish to just disable all File based
    # attachments entirely. In this case, they simply just need to change the
    # global singleton at the start of their program like:
    #
    # import apprise
    # apprise.attachment.AttachFile.location = \
    #       apprise.ContentLocation.INACCESSIBLE
    #
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    response.location = ContentLocation.INACCESSIBLE
    assert response.path is None
    # Downloads just don't work period
    assert response.download() is False

    # File handling (even if image is set to maxium allowable)
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    with mock.patch("os.path.getsize", return_value=AttachBase.max_file_size):
        # It will still work
        assert response.path == path

    # File handling when size is to large
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    with mock.patch(
        "os.path.getsize", return_value=AttachBase.max_file_size + 1
    ):
        # We can't work in this case
        assert response.path is None

    # File handling when image is not available
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    with mock.patch("os.path.isfile", return_value=False):
        # This triggers a full check and will fail the isfile() check
        assert response.path is None

    # The call to AttachBase.path automatically triggers a call to download()
    # but this same is done with a call to AttachBase.name as well.  Above
    # test cases reference 'path' right after instantiation; here we reference
    # 'name'
    response = AppriseAttachment.instantiate(path)
    assert response.name == "apprise-test.gif"
    assert response.path == path
    assert response.mimetype == "image/gif"
    # No mime-type and/or filename over-ride was specified, so therefore it
    # won't show up in the generated URL
    assert re.search(r"[?&]mime=", response.url()) is None
    assert re.search(r"[?&]name=", response.url()) is None

    # continuation to cheking 'name' instead of 'path' first where our call
    # to download() fails
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    with mock.patch("os.path.isfile", return_value=False):
        # This triggers a full check and will fail the isfile() check
        assert response.name is None

    # The call to AttachBase.path automatically triggers a call to download()
    # but this same is done with a call to AttachBase.mimetype as well.  Above
    # test cases reference 'path' right after instantiation; here we reference
    # 'mimetype'
    response = AppriseAttachment.instantiate(path)
    assert response.mimetype == "image/gif"
    assert response.name == "apprise-test.gif"
    assert response.path == path
    # No mime-type and/or filename over-ride was specified, so therefore it
    # won't show up in the generated URL
    assert re.search(r"[?&]mime=", response.url()) is None
    assert re.search(r"[?&]name=", response.url()) is None

    # continuation to cheking 'name' instead of 'path' first where our call
    # to download() fails
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    with mock.patch("os.path.isfile", return_value=False):
        # download() fails so we don't have a mimetpe
        assert response.mimetype is None
        assert response.name is None
        assert response.path is None
        # This triggers a full check and will fail the isfile() check

    # Force a mime-type and new name
    response = AppriseAttachment.instantiate(
        "file://{}?mime={}&name={}".format(path, "image/jpeg", "test.jpeg")
    )
    assert isinstance(response, AttachFile)
    assert response.path == path
    assert response.name == "test.jpeg"
    assert response.mimetype == "image/jpeg"
    assert re.search(r"[?&]mime=image/jpeg", response.url(), re.I)
    assert re.search(r"[?&]name=test\.jpeg", response.url(), re.I)

    # Test hosted configuration and that we can't add a valid file
    aa = AppriseAttachment(location=ContentLocation.HOSTED)
    # No entries defined yet
    assert bool(aa) is False
    assert aa.sync() is False

    # Entry count does not impact sync if told to act that way
    assert aa.sync(abort_if_empty=False) is True
    assert aa.add(path) is False

    response = AppriseAttachment.instantiate(path)
    assert len(response) > 0

    # Get file
    assert response.download()

    # Test the inability to get our file size
    with mock.patch("os.path.getsize", side_effect=(0, OSError)):
        assert len(response) == 0

    # get file again
    assert response.download()
    with mock.patch("os.path.isfile", return_value=True):
        response.cache = True
        with mock.patch("os.path.getsize", side_effect=OSError):
            assert len(response) == 0


def test_attach_file_base64():
    """
    API: AttachFile() with base64 encoding

    """

    # Simple gif test
    path = join(TEST_VAR_DIR, "apprise-test.gif")
    response = AppriseAttachment.instantiate(path)
    assert isinstance(response, AttachFile)
    assert response.name == "apprise-test.gif"
    assert response.mimetype == "image/gif"

    # now test our base64 output
    assert isinstance(response.base64(), str)
    # No encoding if we choose
    assert isinstance(response.base64(encoding=None), bytes)

    # Error cases:
    with (
        mock.patch("os.path.isfile", return_value=False),
        pytest.raises(exception.AppriseFileNotFound),
    ):
        response.base64()

    with mock.patch(
        "builtins.open",
        new_callable=mock.mock_open,
        read_data="mocked file content",
    ) as mock_file:
        mock_file.side_effect = FileNotFoundError
        with pytest.raises(exception.AppriseFileNotFound):
            response.base64()

        mock_file.side_effect = OSError
        with pytest.raises(exception.AppriseDiskIOError):
            response.base64()



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging

from helpers import AppriseURLTester

from apprise.plugins.dingtalk import NotifyDingTalk

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "dingtalk://",
        {
            # No Access Token specified
            "instance": TypeError,
        },
    ),
    (
        "dingtalk://a_bd_/",
        {
            # invalid Access Token
            "instance": TypeError,
        },
    ),
    (
        "dingtalk://12345678",
        {
            # access token
            "instance": NotifyDingTalk,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "dingtalk://1...8",
        },
    ),
    (
        "dingtalk://{}/{}".format("a" * 8, "1" * 14),
        {
            # access token + phone number
            "instance": NotifyDingTalk,
        },
    ),
    (
        "dingtalk://{}/{}/invalid".format("a" * 8, "1" * 3),
        {
            # access token + 2 invalid phone numbers
            "instance": NotifyDingTalk,
        },
    ),
    (
        "dingtalk://{}/?to={}".format("a" * 8, "1" * 14),
        {
            # access token + phone number using 'to'
            "instance": NotifyDingTalk,
        },
    ),
    # Test secret via user@
    (
        "dingtalk://secret@{}/?to={}".format("a" * 8, "1" * 14),
        {
            # access token + phone number using 'to'
            "instance": NotifyDingTalk,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "dingtalk://****@a...a",
        },
    ),
    # Test secret via secret= and token=
    (
        "dingtalk://?token={}&to={}&secret={}".format(
            "b" * 8, "1" * 14, "a" * 15
        ),
        {
            # access token + phone number using 'to'
            "instance": NotifyDingTalk,
            "privacy_url": "dingtalk://****@b...b",
        },
    ),
    # Invalid secret
    (
        "dingtalk://{}/?to={}&secret=_".format("a" * 8, "1" * 14),
        {
            "instance": TypeError,
        },
    ),
    (
        "dingtalk://{}?format=markdown".format("a" * 8),
        {
            # access token
            "instance": NotifyDingTalk,
        },
    ),
    (
        "dingtalk://{}".format("a" * 8),
        {
            "instance": NotifyDingTalk,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "dingtalk://{}".format("a" * 8),
        {
            "instance": NotifyDingTalk,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_dingtalk_urls():
    """NotifyDingTalk() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import dumps, loads

# Disable logging for a cleaner testing output
import logging
import os
from unittest import mock

from helpers import AppriseURLTester
import requests

from apprise import Apprise, AppriseAttachment, NotifyType
from apprise.plugins.smseagle import NotifySMSEagle

logging.disable(logging.CRITICAL)

SMSEAGLE_GOOD_RESPONSE = dumps(
    {"result": {"message_id": "748", "status": "ok"}}
)

SMSEAGLE_BAD_RESPONSE = dumps(
    {
        "result": {
            "error_text": "Wrong parameters",
            "status": "error",
        }
    }
)


# Attachment Directory
TEST_VAR_DIR = os.path.join(os.path.dirname(__file__), "var")

# Our Testing URLs
apprise_url_tests = (
    (
        "smseagle://",
        {
            # No host specified
            "instance": TypeError,
        },
    ),
    (
        "smseagle://:@/",
        {
            # invalid host
            "instance": TypeError,
        },
    ),
    (
        "smseagle://localhost",
        {
            # Just a host provided (no access token)
            "instance": TypeError,
        },
    ),
    (
        "smseagle://%20@localhost",
        {
            # invalid token
            "instance": TypeError,
        },
    ),
    (
        "smseagle://token@localhost/123/",
        {
            # invalid 'to' phone number
            "instance": NotifySMSEagle,
            # Notify will fail because it couldn't send to anyone
            "response": False,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost/@123",
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://tokenb@localhost/%20/%20/",
        {
            # invalid 'to' phone number
            "instance": NotifySMSEagle,
            # Notify will fail because it couldn't send to anyone
            "notify_response": False,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost/",
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost:8080/{}/".format("1" * 11),
        {
            # one phone number will notify ourselves
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://localhost:8080/{}/?token=abc1234".format("1" * 11),
        {
            # pass our token in as an argument
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
            "force_debug": True,
        },
    ),
    # Set priority
    (
        "smseagle://token@localhost/@user/?priority=high",
        {
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    # Support integer value too
    (
        "smseagle://token@localhost/@user/?priority=1",
        {
            "instance": NotifySMSEagle,
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    # Invalid priority
    (
        "smseagle://token@localhost/@user/?priority=invalid",
        {
            # Invalid Priority
            "instance": TypeError,
        },
    ),
    # Invalid priority
    (
        "smseagle://token@localhost/@user/?priority=25",
        {
            # Invalid Priority
            "instance": TypeError,
        },
    ),
    (
        "smseagle://token@localhost:8082/#abcd/",
        {
            # a valid group
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost:8082/#abcd",
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost:8082/@abcd/",
        {
            # a valid contact
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost:8082/@abcd",
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost:8081/contact/",
        {
            # another valid group (without @ symbol)
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagles://****@localhost:8081/@contact",
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost:8082/@/#/,/",
        {
            # Test case where we provide bad data
            "instance": NotifySMSEagle,
            # Our failed response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
            # as a result, we expect a failed notification
            "response": False,
        },
    ),
    (
        "smseagle://token@localhost:8083/@user/",
        {
            # Test case where we get a bad response
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost:8083/@user",
            # Our failed response
            "requests_response_text": SMSEAGLE_BAD_RESPONSE,
            # as a result, we expect a failed notification
            "response": False,
        },
    ),
    (
        "smseagle://token@localhost:8084/@user/",
        {
            # Test case where we get a bad response
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost:8084/@user",
            # Our failed response
            "requests_response_text": None,
            # as a result, we expect a failed notification
            "response": False,
        },
    ),
    (
        "smseagle://token@localhost:8085/@user/",
        {
            # Test case where we get a bad response
            "instance": NotifySMSEagle,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "smseagle://****@localhost:8085/@user",
            # Our failed response (bad json)
            "requests_response_text": "{",
            # as a result, we expect a failed notification
            "response": False,
        },
    ),
    (
        "smseagle://token@localhost:8086/?to={},{}".format("2" * 11, "3" * 11),
        {
            # use get args to acomplish the same thing
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost:8087/?to={},{},{}".format(
            "2" * 11, "3" * 11, "5" * 3
        ),
        {
            # 2 good targets and one invalid one
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost:8088/{}/{}/".format("2" * 11, "3" * 11),
        {
            # If we have from= specified, then all elements take on the
            # to= value
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost/{}".format("3" * 11),
        {
            # use get args to acomplish the same thing (use source instead of
            # from)
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost/{}/{}?batch=True".format(
            "3" * 11, "4" * 11
        ),
        {
            # test batch mode
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost/{}/?flash=yes".format("3" * 11),
        {
            # test flash mode
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost/{}/?test=yes".format("3" * 11),
        {
            # test mode
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagles://token@localhost/{}/{}?status=True".format(
            "3" * 11, "4" * 11
        ),
        {
            # test status switch
            "instance": NotifySMSEagle,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost/{}".format("4" * 11),
        {
            "instance": NotifySMSEagle,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
            # Our response expected server response
            "requests_response_text": SMSEAGLE_GOOD_RESPONSE,
        },
    ),
    (
        "smseagle://token@localhost/{}".format("4" * 11),
        {
            "instance": NotifySMSEagle,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_smseagle_urls():
    """NotifySMSEagle() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_smseagle_edge_cases(mock_post):
    """NotifySMSEagle() Edge Cases."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok
    response.content = SMSEAGLE_GOOD_RESPONSE

    # Prepare Mock
    mock_post.return_value = response

    # Initialize some generic (but valid) tokens
    target = "+1 (555) 987-5432"
    body = "test body"
    title = "My Title"

    aobj = Apprise()
    assert aobj.add(f"smseagles://token@localhost:231/{target}")
    assert len(aobj) == 1
    assert aobj.notify(title=title, body=body)
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    assert details[0][0] == "https://localhost:231/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["params"]["message"] == "My Title\r\ntest body"

    # Reset our mock object
    mock_post.reset_mock()

    aobj = Apprise()
    assert aobj.add(f"smseagles://token@localhost:231/{target}?status=Yes")
    assert len(aobj) == 1
    assert aobj.notify(title=title, body=body)
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    assert details[0][0] == "https://localhost:231/jsonrpc/sms"
    payload = loads(details[1]["data"])
    # Status flag is set
    assert payload["params"]["message"] == "[i] My Title\r\ntest body"


@mock.patch("requests.post")
def test_plugin_smseagle_result_set(mock_post):
    """NotifySMSEagle() Result Sets."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok
    response.content = SMSEAGLE_GOOD_RESPONSE

    # Prepare Mock
    mock_post.return_value = response

    body = "test body"
    title = "My Title"

    aobj = Apprise()
    aobj.add(
        "smseagle://token@10.0.0.112:8080/+12512222222/+12513333333/"
        "12514444444?batch=yes"
    )
    # In a batch mode we can shove them all into 1 call
    assert len(aobj[0]) == 1

    assert aobj.notify(title=title, body=body)

    # If a batch, there is only 1 post
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert "method" in payload
    assert payload["method"] == "sms.send_sms"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "to" in params
    assert len(params["to"].split(",")) == 3

    assert "+12512222222" in params["to"].split(",")
    assert "+12513333333" in params["to"].split(",")
    # The + is not appended
    assert "12514444444" in params["to"].split(",")
    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    # Reset our test and turn batch mode off
    mock_post.reset_mock()

    aobj = Apprise()
    aobj.add(
        "smseagle://token@10.0.0.112:8080/#group/Contact/123456789?batch=no"
    )
    assert len(aobj[0]) == 3

    assert aobj.notify(title=title, body=body)

    # If batch is off then there is a post per entry
    assert mock_post.call_count == 3

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_sms"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "to" in params
    assert len(params["to"].split(",")) == 1
    assert "123456789" in params["to"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    details = mock_post.call_args_list[1]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_togroup"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "groupname" in params
    assert len(params["groupname"].split(",")) == 1
    assert "group" in params["groupname"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    details = mock_post.call_args_list[2]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_tocontact"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "contactname" in params
    assert len(params["contactname"].split(",")) == 1
    assert "Contact" in params["contactname"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    mock_post.reset_mock()

    # Test groups and contact names
    aobj = Apprise()
    aobj.add(
        "smseagle://token@10.0.0.112:8080/513333333/#group1/@contact1/"
        "contact2/12514444444?batch=yes"
    )

    # contacts and numbers can be combined and is calculated in batch response
    assert len(aobj[0]) == 3
    assert aobj.notify(title=title, body=body)

    # There is a unique post to each (group, contact x2, and phone x2)
    # The key is the contacts were grouped here in 1 post each
    assert mock_post.call_count == 3

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_sms"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "to" in params
    assert len(params["to"].split(",")) == 2
    assert "513333333" in params["to"].split(",")
    assert "12514444444" in params["to"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    details = mock_post.call_args_list[1]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_togroup"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "groupname" in params
    assert len(params["groupname"].split(",")) == 1
    assert "group1" in params["groupname"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    details = mock_post.call_args_list[2]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_tocontact"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "contactname" in params
    assert len(params["contactname"].split(",")) == 2
    assert "contact1" in params["contactname"].split(",")
    assert "contact2" in params["contactname"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "My Title\r\ntest body"

    # Validate our information is also placed back into the assembled URL
    assert "/@contact1" in aobj[0].url()
    assert "/@contact2" in aobj[0].url()
    assert "/#group1" in aobj[0].url()
    assert "/513333333" in aobj[0].url()
    assert "/12514444444" in aobj[0].url()


@mock.patch("requests.post")
def test_notify_smseagle_plugin_result_list(mock_post):
    """NotifySMSEagle() Result List Response."""

    okay_response = requests.Request()
    okay_response.status_code = requests.codes.ok
    # We want to test the case where the `result` set returned is a list
    okay_response.content = dumps(
        {"result": [{"message_id": "748", "status": "ok"}]}
    )

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    obj = Apprise.instantiate("smseagle://token@127.0.0.1/12222222/")
    assert isinstance(obj, NotifySMSEagle)

    # We should successfully handle the list
    assert obj.notify("test") is True

    # However if one of the elements in the list is bad
    okay_response.content = dumps(
        {
            "result": [
                {"message_id": "748", "status": "ok"},
                {"message_id": "749", "status": "error"},
            ]
        }
    )

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    # We should now fail
    assert obj.notify("test") is False


@mock.patch("requests.post")
def test_notify_smseagle_plugin_attachments(mock_post):
    """NotifySMSEagle() Attachments."""

    okay_response = requests.Request()
    okay_response.status_code = requests.codes.ok
    okay_response.content = SMSEAGLE_GOOD_RESPONSE

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    obj = Apprise.instantiate(
        "smseagle://token@10.0.0.112:8080/+12512222222/+12513333333/"
        "12514444444?batch=no"
    )
    assert isinstance(obj, NotifySMSEagle)

    # Test Valid Attachment
    path = os.path.join(TEST_VAR_DIR, "apprise-test.gif")
    attach = AppriseAttachment(path)
    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=attach,
        )
        is True
    )

    # Test invalid attachment
    path = os.path.join(TEST_VAR_DIR, "/invalid/path/to/an/invalid/file.jpg")
    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=path,
        )
        is False
    )

    path = (
        os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
        os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
        os.path.join(TEST_VAR_DIR, "apprise-test.gif"),
    )
    attach = AppriseAttachment(path)

    # Return our good configuration
    mock_post.side_effect = None
    mock_post.return_value = okay_response
    with mock.patch("builtins.open", side_effect=OSError()):
        # We can't send the message we can't open the attachment for reading
        assert (
            obj.notify(
                body="body",
                title="title",
                notify_type=NotifyType.INFO,
                attach=attach,
            )
            is False
        )

    # test the handling of our batch modes
    obj = Apprise.instantiate(
        "smseagle://token@10.0.0.112:8080/+12512222222/+12513333333/"
        "12514444444?batch=yes"
    )
    assert isinstance(obj, NotifySMSEagle)

    # Now send an attachment normally without issues
    mock_post.reset_mock()
    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=attach,
        )
        is True
    )

    # Verify we posted upstream
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_sms"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "to" in params
    assert len(params["to"].split(",")) == 3
    assert "+12512222222" in params["to"].split(",")
    assert "+12513333333" in params["to"].split(",")
    assert "12514444444" in params["to"].split(",")

    assert params.get("message_type") == "mms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "title\r\nbody"

    # Verify our attachments are in place
    assert "attachments" in params
    assert isinstance(params["attachments"], list)
    assert len(params["attachments"]) == 3
    for entry in params["attachments"]:
        assert "content" in entry
        assert "content_type" in entry
        assert entry.get("content_type").startswith("image/")

    # Reset our mock object
    mock_post.reset_mock()

    # test the handling of our batch modes
    obj = Apprise.instantiate("smseagle://token@10.0.0.112:8080/513333333/")
    assert isinstance(obj, NotifySMSEagle)

    # Unsupported (non image types are not sent)
    attach = os.path.join(TEST_VAR_DIR, "apprise-test.mp4")
    assert (
        obj.notify(
            body="body",
            title="title",
            notify_type=NotifyType.INFO,
            attach=attach,
        )
        is True
    )

    # Verify we still posted upstream
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    assert details[0][0] == "http://10.0.0.112:8080/jsonrpc/sms"
    payload = loads(details[1]["data"])
    assert payload["method"] == "sms.send_sms"

    assert "params" in payload
    assert isinstance(payload["params"], dict)
    params = payload["params"]
    assert "to" in params
    assert len(params["to"].split(",")) == 1
    assert "513333333" in params["to"].split(",")

    assert params.get("message_type") == "sms"
    assert params.get("responsetype") == "extended"
    assert params.get("access_token") == "token"
    assert params.get("highpriority") == 0
    assert params.get("flash") == 0
    assert params.get("test") == 0
    assert params.get("unicode") == 1
    assert params.get("message") == "title\r\nbody"

    # No attachments were added
    assert "attachments" not in params



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import dumps

# Disable logging for a cleaner testing output
import logging
import os
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

import apprise
from apprise.plugins.pushover import NotifyPushover, PushoverPriority

logging.disable(logging.CRITICAL)

# Attachment Directory
TEST_VAR_DIR = os.path.join(os.path.dirname(__file__), "var")

# Our Testing URLs
apprise_url_tests = (
    (
        "pover://",
        {
            "instance": TypeError,
        },
    ),
    # bad url
    (
        "pover://:@/",
        {
            "instance": TypeError,
        },
    ),
    # APIkey; no user
    (
        "pover://%s" % ("a" * 30),
        {
            "instance": TypeError,
        },
    ),
    # API Key + custom sound setting
    (
        "pover://{}@{}?sound=mysound".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + valid alternate sound picked
    (
        "pover://{}@{}?sound=spacealarm".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + valid url_title with url
    (
        "pover://{}@{}?url=my-url&url_title=title".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User
    (
        "pover://{}@{}".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # don't include an image by default
            "include_image": False,
        },
    ),
    # API Key + Valid User + 1 Device
    (
        "pover://{}@{}/DEVICE".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 1 Device (via to=)
    (
        "pover://{}@{}?to=DEVICE".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 2 Devices
    (
        "pover://{}@{}/DEVICE1/Device-with-dash/".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "pover://u...u@a...a",
        },
    ),
    # API Key + Valid User + invalid device
    (
        "pover://{}@{}/{}/".format("u" * 30, "a" * 30, "d" * 30),
        {
            "instance": NotifyPushover,
            # Notify will return False since there is a bad device in our list
            "response": False,
        },
    ),
    # API Key + Valid User + device + invalid device
    (
        "pover://{}@{}/DEVICE1/{}/".format("u" * 30, "a" * 30, "d" * 30),
        {
            "instance": NotifyPushover,
            # Notify will return False since there is a bad device in our list
            "response": False,
        },
    ),
    # API Key + Valid User + 1 Group key (percent-encoded)
    (
        "pover://{}@{}/%23{}".format("u" * 30, "a" * 30, "g" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 1 Group key (literal #)
    (
        "pover://{}@{}/#{}/".format("u" * 30, "a" * 30, "g" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + Group key (via to= percent-encoded)
    (
        "pover://{}@{}?to=%23{}".format("u" * 30, "a" * 30, "g" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 2 Group keys (percent-encoded)
    (
        "pover://{}@{}/%23{}/%23{}".format(
            "u" * 30, "a" * 30, "g" * 30, "h" * 30
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 2 Group keys (literal #)
    (
        "pover://{}@{}/#{}/#{}/".format(
            "u" * 30, "a" * 30, "g" * 30, "h" * 30
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 1 Device + 1 Group key (percent-encoded)
    (
        "pover://{}@{}/DEVICE/%23{}".format("u" * 30, "a" * 30, "g" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + 1 Device + 1 Group key (literal #)
    (
        "pover://{}@{}/DEVICE/#{}".format("u" * 30, "a" * 30, "g" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + Valid User + invalid group (contains invalid chars)
    (
        "pover://{}@{}/%23invalid-group/".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # Notify will return False since the group key is invalid
            # (contains a dash which is not alphanumeric)
            "response": False,
        },
    ),
    # API Key + priority setting
    (
        "pover://{}@{}?priority=high".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + priority setting + html mode
    (
        "pover://{}@{}?priority=high&format=html".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + priority setting + markdown mode
    (
        "pover://{}@{}?priority=high&format=markdown".format(
            "u" * 30, "a" * 30
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + invalid priority setting
    (
        "pover://{}@{}?priority=invalid".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency(2) priority setting
    (
        "pover://{}@{}?priority=emergency".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency(2) priority setting (via numeric value
    (
        "pover://{}@{}?priority=2".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency priority setting with retry and expire
    (
        "pover://{}@{}?priority=emergency&{}&{}".format(
            "u" * 30, "a" * 30, "retry=30", "expire=300"
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency priority setting with text retry
    (
        "pover://{}@{}?priority=emergency&{}&{}".format(
            "u" * 30, "a" * 30, "retry=invalid", "expire=300"
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency priority setting with text expire
    (
        "pover://{}@{}?priority=emergency&{}&{}".format(
            "u" * 30, "a" * 30, "retry=30", "expire=invalid"
        ),
        {
            "instance": NotifyPushover,
        },
    ),
    # API Key + emergency priority setting with invalid expire
    (
        "pover://{}@{}?priority=emergency&{}".format(
            "u" * 30, "a" * 30, "expire=100000"
        ),
        {
            "instance": TypeError,
        },
    ),
    # API Key + emergency priority setting with invalid retry
    (
        "pover://{}@{}?priority=emergency&{}".format(
            "u" * 30, "a" * 30, "retry=15"
        ),
        {
            "instance": TypeError,
        },
    ),
    # API Key + priority setting (empty)
    (
        "pover://{}@{}?priority=".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
        },
    ),
    (
        "pover://{}@{}".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "pover://{}@{}".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "pover://{}@{}".format("u" * 30, "a" * 30),
        {
            "instance": NotifyPushover,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_pushover_urls():
    """NotifyPushover() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_pushover_attachments(mock_post, tmpdir):
    """NotifyPushover() Attachment Checks."""

    # Initialize some generic (but valid) tokens
    user_key = "u" * 30
    api_token = "a" * 30

    # Prepare a good response
    response = mock.Mock()
    response.content = dumps(
        {"status": 1, "request": "647d2300-702c-4b38-8b2f-d56326ae460b"}
    )
    response.status_code = requests.codes.ok

    # Prepare a bad response
    bad_response = mock.Mock()
    bad_response.content = dumps(
        {"status": 1, "request": "647d2300-702c-4b38-8b2f-d56326ae460b"}
    )
    bad_response.status_code = requests.codes.internal_server_error

    # Assign our good response
    mock_post.return_value = response

    # prepare our attachment
    attach = apprise.AppriseAttachment(
        os.path.join(TEST_VAR_DIR, "apprise-test.gif")
    )

    # Instantiate our object
    obj = apprise.Apprise.instantiate(f"pover://{user_key}@{api_token}/")
    assert isinstance(obj, NotifyPushover)

    # Test our attachment
    assert obj.notify(body="test", attach=attach) is True

    # Test our call count
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == "https://api.pushover.net/1/messages.json"
    )

    # Reset our mock object for multiple tests
    mock_post.reset_mock()

    # Test multiple attachments
    assert attach.add(os.path.join(TEST_VAR_DIR, "apprise-test.gif"))
    assert obj.notify(body="test", attach=attach) is True

    # Test our call count
    assert mock_post.call_count == 2
    assert (
        mock_post.call_args_list[0][0][0]
        == "https://api.pushover.net/1/messages.json"
    )
    assert (
        mock_post.call_args_list[1][0][0]
        == "https://api.pushover.net/1/messages.json"
    )

    # Reset our mock object for multiple tests
    mock_post.reset_mock()

    image = tmpdir.mkdir("pover_image").join("test.jpg")
    image.write("a" * NotifyPushover.attach_max_size_bytes)

    attach = apprise.AppriseAttachment.instantiate(str(image))
    assert obj.notify(body="test", attach=attach) is True

    # Test our call count
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == "https://api.pushover.net/1/messages.json"
    )

    # Reset our mock object for multiple tests
    mock_post.reset_mock()

    # Add 1 more byte to the file (putting it over the limit)
    image.write("a" * (NotifyPushover.attach_max_size_bytes + 1))

    attach = apprise.AppriseAttachment.instantiate(str(image))
    assert obj.notify(body="test", attach=attach) is False

    # Test our call count
    assert mock_post.call_count == 0

    # Test case when file is missing
    attach = apprise.AppriseAttachment.instantiate(
        f"file://{image!s}?cache=False"
    )
    os.unlink(str(image))
    assert obj.notify(body="body", title="title", attach=attach) is False

    # Test our call count
    assert mock_post.call_count == 0

    # Test unsuported files:
    image = tmpdir.mkdir("pover_unsupported").join("test.doc")
    image.write("a" * 256)
    attach = apprise.AppriseAttachment.instantiate(str(image))

    # Content is silently ignored
    assert obj.notify(body="test", attach=attach) is True

    # prepare our attachment
    attach = apprise.AppriseAttachment(
        os.path.join(TEST_VAR_DIR, "apprise-test.gif")
    )

    # Throw an exception on the first call to requests.post()
    for side_effect in (requests.RequestException(), OSError(), bad_response):
        mock_post.side_effect = [side_effect, side_effect]

        # We'll fail now because of our error handling
        assert obj.send(body="test", attach=attach) is False

        # Same case without an attachment
        assert obj.send(body="test") is False


@mock.patch("requests.post")
def test_plugin_pushover_edge_cases(mock_post):
    """NotifyPushover() Edge Cases."""

    # No token
    with pytest.raises(TypeError):
        NotifyPushover(token=None)

    # Initialize some generic (but valid) tokens
    token = "a" * 30
    user_key = "u" * 30

    invalid_device = "d" * 35

    # Support strings
    devices = f"device1,device2,,,,{invalid_device}"

    # Prepare Mock
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.ok

    # No webhook id specified
    with pytest.raises(TypeError):
        NotifyPushover(user_key=user_key, webhook_id=None)

    obj = NotifyPushover(user_key=user_key, token=token, targets=devices)
    assert isinstance(obj, NotifyPushover)
    # Our invalid device is ignored; 2 valid devices remain
    assert len(obj.devices) == 2
    assert len(obj.groups) == 0

    # We notify the 2 devices loaded
    assert (
        obj.notify(
            body="body", title="title", notify_type=apprise.NotifyType.INFO
        )
        is True
    )

    obj = NotifyPushover(user_key=user_key, token=token)
    assert isinstance(obj, NotifyPushover)
    # Default is to send to all devices (sentinel placed in devices list)
    assert len(obj.devices) == 1
    assert len(obj.groups) == 0
    assert len(obj) == 1

    # This call succeeds because all of the devices are valid
    assert (
        obj.notify(
            body="body", title="title", notify_type=apprise.NotifyType.INFO
        )
        is True
    )

    obj = NotifyPushover(user_key=user_key, token=token, targets=set())
    assert isinstance(obj, NotifyPushover)
    # Default is to send to all devices (sentinel placed in devices list)
    assert len(obj.devices) == 1
    assert len(obj.groups) == 0
    assert len(obj) == 1

    # Group targets
    group_key = "g" * 30
    obj = NotifyPushover(
        user_key=user_key, token=token, targets=f"#{group_key}"
    )
    assert isinstance(obj, NotifyPushover)
    assert len(obj.devices) == 0
    assert len(obj.groups) == 1
    assert obj.groups[0] == group_key
    assert len(obj) == 1

    # Verify notification to a group succeeds (separate API call per group)
    assert (
        obj.notify(
            body="body", title="title", notify_type=apprise.NotifyType.INFO
        )
        is True
    )

    # Mix of device and group → 2 HTTP calls (1 device batch + 1 group)
    obj = NotifyPushover(
        user_key=user_key, token=token, targets=["device1", f"#{group_key}"]
    )
    assert isinstance(obj, NotifyPushover)
    assert len(obj.devices) == 1
    assert len(obj.groups) == 1
    assert len(obj) == 2

    # Invalid group key (contains dash — not alphanumeric)
    obj = NotifyPushover(
        user_key=user_key, token=token, targets="#invalid-group"
    )
    assert isinstance(obj, NotifyPushover)
    # Invalid target is rejected; both lists empty → send() returns False
    assert len(obj.devices) == 0
    assert len(obj.groups) == 0

    # Always 1 is returned as minimum
    assert len(obj) == 1

    # No User Key specified
    with pytest.raises(TypeError):
        NotifyPushover(user_key=None, token="abcd")

    # No Access Token specified
    with pytest.raises(TypeError):
        NotifyPushover(user_key="abcd", token=None)

    with pytest.raises(TypeError):
        NotifyPushover(user_key="abcd", token="  ")


@mock.patch("requests.post")
def test_plugin_pushover_config_files(mock_post):
    """NotifyPushover() Config File Cases."""
    content = """
    urls:
      - pover://USER@TOKEN:
          - priority: -2
            tag: pushover_int low
          - priority: "-2"
            tag: pushover_str_int low
          - priority: low
            tag: pushover_str low

          # This will take on normal (default) priority
          - priority: invalid
            tag: pushover_invalid

      - pover://USER2@TOKEN2:
          - priority: 2
            tag: pushover_int emerg
          - priority: "2"
            tag: pushover_str_int emerg
          - priority: emergency
            tag: pushover_str emerg
    """

    # Prepare Mock
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.ok

    # Create ourselves a config object
    ac = apprise.AppriseConfig()
    assert ac.add_config(content=content) is True

    aobj = apprise.Apprise()

    # Add our configuration
    aobj.add(ac)

    # We should be able to read our 7 servers from that
    # 3x low
    # 3x emerg
    # 1x invalid (so takes on normal priority)
    assert len(ac.servers()) == 7
    assert len(aobj) == 7
    assert len(list(aobj.find(tag="low"))) == 3
    for s in aobj.find(tag="low"):
        assert s.priority == PushoverPriority.LOW

    assert len(list(aobj.find(tag="emerg"))) == 3
    for s in aobj.find(tag="emerg"):
        assert s.priority == PushoverPriority.EMERGENCY

    assert len(list(aobj.find(tag="pushover_str"))) == 2
    assert len(list(aobj.find(tag="pushover_str_int"))) == 2
    assert len(list(aobj.find(tag="pushover_int"))) == 2

    assert len(list(aobj.find(tag="pushover_invalid"))) == 1
    assert (
        next(aobj.find(tag="pushover_invalid")).priority
        == PushoverPriority.NORMAL
    )

    # Notifications work
    # We test 'pushover_str_int' and 'low' which only matches 1 end point
    assert (
        aobj.notify(
            title="title", body="body", tag=[("pushover_str_int", "low")]
        )
        is True
    )

    # Notify everything loaded
    assert aobj.notify(title="title", body="body") is True


@mock.patch("requests.post")
def test_plugin_pushover_group_request_exception(mock_post):
    """NotifyPushover() group RequestException must not raise KeyError."""
    user_key = "u" * 30
    token = "a" * 30
    group_key = "g" * 30

    mock_post.side_effect = requests.RequestException("connection error")

    obj = NotifyPushover(
        user_key=user_key, token=token, targets=f"#{group_key}"
    )
    assert isinstance(obj, NotifyPushover)
    assert len(obj.groups) == 1
    assert len(obj.devices) == 0

    # Must return False cleanly -- no KeyError from payload["device"]
    assert obj.send(body="test") is False


@mock.patch("requests.post")
def test_plugin_pushover_url_roundtrip(mock_post):
    """NotifyPushover() url() must preserve sound, url, and url_title."""
    from apprise.plugins.pushover import NotifyPushover

    user_key = "u" * 30
    token = "a" * 30

    mock_post.return_value = mock.Mock()
    mock_post.return_value.status_code = requests.codes.ok

    # Instantiate with all three round-trip fields set
    obj = NotifyPushover(
        user_key=user_key,
        token=token,
        sound="bike",
        supplemental_url="https://example.com",
        supplemental_url_title="Click Here",
    )
    assert isinstance(obj, NotifyPushover)
    assert obj.sound == "bike"
    assert obj.supplemental_url == "https://example.com"
    assert obj.supplemental_url_title == "Click Here"

    # Round-trip via url() -> parse_url() -> re-instantiate
    generated_url = obj.url()
    assert "sound=bike" in generated_url
    assert "url=" in generated_url
    assert "url_title=" in generated_url

    parsed = NotifyPushover.parse_url(generated_url)
    assert parsed is not None
    assert parsed["sound"] == "bike"
    assert parsed["supplemental_url"] == "https://example.com"
    assert parsed["supplemental_url_title"] == "Click Here"

    obj2 = NotifyPushover(**parsed)
    assert obj2.sound == "bike"
    assert obj2.supplemental_url == "https://example.com"
    assert obj2.supplemental_url_title == "Click Here"


def test_plugin_pushover_parse_url_title_unquote():
    """NotifyPushover() parse_url() unquote url_title like other fields."""
    user_key = "u" * 30
    token = "a" * 30

    # url_title with percent-encoded spaces and special chars
    url = "pover://{}@{}/?url_title=Hello%20World".format(user_key, token)
    parsed = NotifyPushover.parse_url(url)
    assert parsed is not None
    # Must be decoded, not raw percent-encoded
    assert parsed["supplemental_url_title"] == "Hello World"

    # Spacing is fine too
    url = "pover://{}@{}/?url_title=Hello World".format(user_key, token)
    parsed = NotifyPushover.parse_url(url)
    assert parsed is not None
    # Must be decoded, not raw percent-encoded
    assert parsed["supplemental_url_title"] == "Hello World"


@mock.patch("requests.post")
def test_plugin_pushover_attach_memory(mock_post):
    """Regression: AttachMemory must be sendable without OSError."""
    from apprise.attachment.memory import AttachMemory

    user_key = "u" * 30
    api_token = "a" * 30

    response = mock.Mock()
    response.content = dumps(
        {"status": 1, "request": "647d2300-702c-4b38-8b2f-d56326ae460b"}
    )
    response.status_code = requests.codes.ok
    mock_post.return_value = response

    obj = apprise.Apprise.instantiate(f"pover://{user_key}@{api_token}/")
    assert isinstance(obj, NotifyPushover)

    mem = AttachMemory(
        content=b"<html><body><h1>Test</h1></body></html>",
        name="report.html",
        mimetype="text/html",
    )

    assert obj.notify(body="Test", attach=mem) is True
    assert mock_post.call_count == 1



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import ctypes
from importlib import reload

# Disable logging for a cleaner testing output
import logging
import os
import sys
from unittest import mock

from helpers import environ
import pytest

from apprise import locale

logging.disable(logging.CRITICAL)


def test_apprise_trans():
    """
    API: Test apprise locale object
    """
    lazytrans = locale.LazyTranslation("Token")
    assert str(lazytrans) == "Token"


@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
def test_apprise_trans_gettext_init():
    """
    API: Handle gettext
    """
    # Toggle
    locale.GETTEXT_LOADED = False

    # Objects can still be created
    al = locale.AppriseLocale()

    with al.lang_at("en") as _:
        # functions still behave as normal
        assert _ is None

    # Restore the object
    locale.GETTEXT_LOADED = True


@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
@mock.patch("gettext.translation")
@mock.patch("locale.getlocale")
def test_apprise_trans_gettext_translations(
    mock_getlocale, mock_gettext_trans
):
    """
    API: Apprise() Gettext translations

    """

    # Set- our gettext.locale() return value
    mock_getlocale.return_value = ("en_US", "UTF-8")

    mock_gettext_trans.side_effect = FileNotFoundError()

    # This throws internally but we handle it gracefully
    al = locale.AppriseLocale()

    with al.lang_at("en"):
        # functions still behave as normal
        pass

    # This throws internally but we handle it gracefully
    locale.AppriseLocale(language="fr")


@pytest.mark.skipif(hasattr(ctypes, "windll"), reason="Unique Nux test cases")
@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
@mock.patch("locale.getlocale")
def test_apprise_trans_gettext_lang_at(mock_getlocale):
    """
    API: Apprise() Gettext lang_at

    """

    # Set- our gettext.locale() return value
    mock_getlocale.return_value = ("en_CA", "UTF-8")

    # This throws internally but we handle it gracefully
    al = locale.AppriseLocale()

    # Edge Cases
    assert al.add("en", set_default=False) is True
    assert al.add("en", set_default=True) is True

    with al.lang_at("en"):
        # functions still behave as normal
        pass

    # This throws internally but we handle it gracefully
    locale.AppriseLocale(language="fr")

    with al.lang_at("en") as _:
        # functions still behave as normal
        assert callable(_)

    with al.lang_at("es") as _:
        # functions still behave as normal
        assert callable(_)

    with al.lang_at("fr") as _:
        # functions still behave as normal
        assert callable(_)

    # Test our initialization when our fallback is a language we do
    # not have. This is only done to test edge cases when for whatever
    # reason the person who set up apprise does not have the languages
    # installed.
    fallback = locale.AppriseLocale._default_language
    mock_getlocale.return_value = None

    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
        # Our default language
        locale.AppriseLocale._default_language = "zz"

        # We will detect the zz since there were no environment variables to
        # help us otherwise
        assert locale.AppriseLocale.detect_language() is None
        al = locale.AppriseLocale()

        # No Language could be set becuause no locale directory exists for this
        assert al.lang is None

        with al.lang_at(None) as _:
            # functions still behave as normal
            assert callable(_)

        with al.lang_at("en") as _:
            # functions still behave as normal
            assert callable(_)

        with al.lang_at("es") as _:
            # functions still behave as normal
            assert callable(_)

        with al.lang_at("fr") as _:
            # functions still behave as normal
            assert callable(_)

        # We can still perform simple lookups; they access a dummy wrapper:
        assert al.gettext("test") == "test"

    with environ("LANGUAGE", "LC_CTYPE", LC_ALL="C.UTF-8", LANG="en_CA"):
        # the UTF-8 entry is skipped over
        locale.AppriseLocale._default_language = "fr"

        # We will detect the english language (found in the LANG= environment
        # variable which over-rides the _default
        assert locale.AppriseLocale.detect_language() == "en"
        al = locale.AppriseLocale()
        assert al.lang == "en"
        assert al.gettext("test") == "test"

        # Test case with set_default set to False (so we're still set to 'fr')
        assert al.add("zy", set_default=False) is False
        assert al.gettext("test") == "test"

        al.add("ab", set_default=True)
        assert al.gettext("test") == "test"

        assert al.add("zy", set_default=False) is False
    locale.AppriseLocale._default_language = fallback


@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
def test_apprise_trans_add():
    """
    API: Apprise() Gettext add

    """

    # This throws internally but we handle it gracefully
    al = locale.AppriseLocale()
    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
        # English is the default/fallback type
        assert al.add("en") is True

    al = locale.AppriseLocale()
    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", LANG="C.UTF-8"):
        # Test English Environment
        assert al.add("en") is True

    al = locale.AppriseLocale()
    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", LANG="en_CA.UTF-8"):
        # Test English Environment
        assert al.add("en") is True

        # Double add (copy of above) to access logic that prevents adding it
        # again
        assert al.add("en") is True

    # Invalid Language
    assert al.add("bad") is False


@pytest.mark.skipif(
    not hasattr(ctypes, "windll"), reason="Unique Windows test cases"
)
@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
@mock.patch("locale.getlocale")
def test_apprise_trans_windows_users_win(mock_getlocale):
    """
    API: Apprise() Windows Locale Testing (Win version)

    """

    # Set- our gettext.locale() return value
    mock_getlocale.return_value = ("fr_CA", "UTF-8")

    with mock.patch(
        "ctypes.windll.kernel32.GetUserDefaultUILanguage"
    ) as ui_lang:
        # 4105 = en_CA
        ui_lang.return_value = 4105

        with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
            # Our default language
            locale.AppriseLocale._default_language = "zz"

            # We will pick up the windll module and detect english
            assert locale.AppriseLocale.detect_language() == "en"

        # The below accesses the windows fallback code
        with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", LANG="es_AR"):
            # Environment Variable Trumps
            assert locale.AppriseLocale.detect_language() == "es"

        # No environment variable, then the Windows environment is used
        with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
            # Windows Environment
            assert locale.AppriseLocale.detect_language() == "en"

        assert (
            locale.AppriseLocale.detect_language(detect_fallback=False) is None
        )

        # 0 = IndexError
        ui_lang.return_value = 0
        with environ("LANGUAGE", "LANG", "LC_ALL", "LC_CTYPE"):
            # We fall back to posix locale
            assert locale.AppriseLocale.detect_language() == "fr"


@pytest.mark.skipif(hasattr(ctypes, "windll"), reason="Unique Nux test cases")
@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
@mock.patch("locale.getlocale")
def test_apprise_trans_windows_users_nux(mock_getlocale):
    """
    API: Apprise() Windows Locale Testing (Nux version)

    """

    # Set- our gettext.locale() return value
    mock_getlocale.return_value = ("fr_CA", "UTF-8")

    # Emulate a windows environment
    windll = mock.Mock()
    ctypes.windll = windll

    # 4105 = en_CA
    windll.kernel32.GetUserDefaultUILanguage.return_value = 4105

    # Store default value to not break other tests
    default_language = locale.AppriseLocale._default_language

    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
        # Our default language
        locale.AppriseLocale._default_language = "zz"

        # We will pick up the windll module and detect english
        assert locale.AppriseLocale.detect_language() == "en"

    # The below accesses the windows fallback code
    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", LANG="es_AR"):
        # Environment Variable Trumps
        assert locale.AppriseLocale.detect_language() == "es"

    # No environment variable, then the Windows environment is used
    with environ("LANGUAGE", "LC_ALL", "LC_CTYPE", "LANG"):
        # Windows Environment
        assert locale.AppriseLocale.detect_language() == "en"

    assert locale.AppriseLocale.detect_language(detect_fallback=False) is None

    # 0 = IndexError
    windll.kernel32.GetUserDefaultUILanguage.return_value = 0
    with environ("LANGUAGE", "LANG", "LC_ALL", "LC_CTYPE"):
        # We fall back to posix locale
        assert locale.AppriseLocale.detect_language() == "fr"

    del ctypes.windll

    # Restore default value
    locale.AppriseLocale._default_language = default_language


@pytest.mark.skipif(sys.platform == "win32", reason="Unique Nux test cases")
@mock.patch("locale.getlocale")
def test_detect_language_using_env(mock_getlocale):
    """Test the reading of information from an environment variable."""

    # Set- our gettext.locale() return value
    mock_getlocale.return_value = ("en_CA", "UTF-8")

    # The below accesses the windows fallback code and fail
    # then it will resort to the environment variables.
    with environ("LANG", "LANGUAGE", "LC_ALL", "LC_CTYPE"):
        # Language can now be detected in this case
        assert isinstance(locale.AppriseLocale.detect_language(), str)

    # Detect French language.
    with environ("LANGUAGE", "LC_ALL", LC_CTYPE="garbage", LANG="fr_CA"):
        assert locale.AppriseLocale.detect_language() == "fr"

    # The following unsets all environment variables and sets LC_CTYPE
    # This was causing Python 2.7 to internally parse UTF-8 as an invalid
    # locale and throw an uncaught ValueError; Python v2 support has been
    # dropped, but just to ensure this issue does not come back, we keep
    # this test:
    with environ(*list(os.environ.keys()), LC_CTYPE="UTF-8"):
        assert isinstance(locale.AppriseLocale.detect_language(), str)

    # Test with absolutely no environment variables what-so-ever
    with environ(*list(os.environ.keys())):
        assert isinstance(locale.AppriseLocale.detect_language(), str)

    # Handle case where getlocale() can't be detected
    mock_getlocale.return_value = None
    with environ("LC_ALL", "LC_CTYPE", "LANG", "LANGUAGE"):
        assert locale.AppriseLocale.detect_language() is None

    mock_getlocale.return_value = (None, None)
    with environ("LC_ALL", "LC_CTYPE", "LANG", "LANGUAGE"):
        assert locale.AppriseLocale.detect_language() is None

    # if detect_language and windows env fail us, then we don't
    # set up a default language on first load
    locale.AppriseLocale()


@pytest.mark.skipif("gettext" not in sys.modules, reason="Requires gettext")
def test_apprise_trans_gettext_missing(tmpdir):
    """Verify we can still operate without the gettext library."""

    # remove gettext from our system enviroment
    del sys.modules["gettext"]

    # Make our new path to a fake gettext (used to over-ride real one)
    # have it fail right out of the gate
    gettext_dir = tmpdir.mkdir("gettext")
    gettext_dir.join("__init__.py").write("")
    gettext_dir.join("gettext.py").write("""raise ImportError()""")

    # Update our path to point path to head
    sys.path.insert(0, str(gettext_dir))

    # reload our module (forcing the import error when it tries to load gettext
    reload(sys.modules["apprise.locale"])
    from apprise import locale

    assert locale.GETTEXT_LOADED is False

    # Now roll our changes back
    sys.path.pop(0)

    # Reload again (reverting back)
    reload(sys.modules["apprise.locale"])
    from apprise import locale

    assert locale.GETTEXT_LOADED is True


@mock.patch("gettext.translation")
def test_apprise_locale_add_existing_language(mock_translation):
    """Re-add an existing language to hit fallback mapping path."""
    # Return a fake gettext object with a valid gettext() method
    dummy = mock.Mock()
    dummy.gettext = lambda x: f"tr:{x}"
    mock_translation.return_value = dummy

    al = locale.AppriseLocale()
    assert al.add("en", set_default=True) is True
    assert al.add("en", set_default=False) is True


@mock.patch("gettext.translation")
@mock.patch("locale.getlocale")
def test_apprise_trans_successful_translation(
    mock_getlocale, mock_translation
):
    """Trigger logger.trace by mocking successful gettext.translation()"""
    mock_getlocale.return_value = ("en_CA", "UTF-8")

    dummy_translation = mock.Mock()
    dummy_translation.gettext = lambda x: f"tr:{x}"
    mock_translation.return_value = dummy_translation

    al = locale.AppriseLocale()
    assert al.add("en") is True
    assert al.gettext("test") == "tr:test"



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import requests

import apprise
from apprise import NotifyType
from apprise.plugins.dapnet import DapnetPriority, NotifyDapnet

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "dapnet://",
        {
            # We failed to identify any valid authentication
            "instance": TypeError,
        },
    ),
    (
        "dapnet://:@/",
        {
            # We failed to identify any valid authentication
            "instance": TypeError,
        },
    ),
    (
        "dapnet://user:pass",
        {
            # No call-sign specified
            "instance": TypeError,
        },
    ),
    (
        "dapnet://user@host",
        {
            # No password specified
            "instance": TypeError,
        },
    ),
    (
        "dapnet://user:pass@{}".format("DF1ABC"),
        {
            # valid call sign
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}/{}".format("DF1ABC", "DF1DEF"),
        {
            # valid call signs
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@DF1ABC-1/DF1ABC/DF1ABC-15",
        {
            # valid call signs; but a few are duplicates;
            # at the end there will only be 1 entry
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
            # Our expected url(privacy=True) startswith() response:
            # Note that only 1 entry is saved (as other 2 are duplicates)
            "privacy_url": "dapnet://user:****@D...C?",
        },
    ),
    (
        "dapnet://user:pass@?to={},{}".format("DF1ABC", "DF1DEF"),
        {
            # support the to= argument
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}?priority=normal".format("DF1ABC"),
        {
            # valid call sign with priority
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}?priority=em&batch=false".format(
            "/".join(["DF1ABC", "0A1DEF"])
        ),
        {
            # valid call sign with priority (emergency) + no batch
            # transmissions
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}?priority=invalid".format("DF1ABC"),
        {
            # invalid priority
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}?txgroups=dl-all,all".format("DF1ABC"),
        {
            # valid call sign with two transmitter groups
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}?txgroups=invalid".format("DF1ABC"),
        {
            # valid call sign with invalid transmitter group
            "instance": NotifyDapnet,
            "requests_response_code": requests.codes.created,
        },
    ),
    (
        "dapnet://user:pass@{}/{}".format("abcdefghi", "a"),
        {
            # invalid call signs
            "instance": NotifyDapnet,
            "notify_response": False,
        },
    ),
    # Edge cases
    (
        "dapnet://user:pass@{}".format("DF1ABC"),
        {
            "instance": NotifyDapnet,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "dapnet://user:pass@{}".format("DF1ABC"),
        {
            "instance": NotifyDapnet,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_dapnet_urls():
    """NotifyDapnet() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_dapnet_edge_cases(mock_post):
    """NotifyDapnet() Edge Cases."""
    # Prepare Mock
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.created

    # test the handling of our batch modes
    obj = apprise.Apprise.instantiate(
        "dapnet://user:pass@{}?batch=yes".format(
            "/".join(["DF1ABC", "DF1DEF"])
        )
    )
    assert isinstance(obj, NotifyDapnet)

    # objects will be combined into a single post in batch mode
    assert len(obj) == 1

    # Force our batch to break into separate messages
    obj.default_batch_size = 1

    # We'll send 2 messages now
    assert len(obj) == 2

    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )
    assert mock_post.call_count == 2


@mock.patch("requests.post")
def test_plugin_dapnet_config_files(mock_post):
    """NotifyDapnet() Config File Cases."""
    content = """
    urls:
      - dapnet://user:pass@DF1ABC:
          - priority: 0
            tag: dapnet_int normal
          - priority: "0"
            tag: dapnet_str_int normal
          - priority: normal
            tag: dapnet_str normal

          # This will take on normal (default) priority
          - priority: invalid
            tag: dapnet_invalid

      - dapnet://user1:pass2@DF1ABC:
          - priority: 1
            tag: dapnet_int emerg
          - priority: "1"
            tag: dapnet_str_int emerg
          - priority: emergency
            tag: dapnet_str emerg
    """

    # Prepare Mock
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.created

    # Create ourselves a config object
    ac = apprise.AppriseConfig()
    assert ac.add_config(content=content) is True

    aobj = apprise.Apprise()

    # Add our configuration
    aobj.add(ac)

    # We should be able to read our 7 servers from that
    # 4x normal (invalid + 3 exclusivly specified to be so)
    # 3x emerg
    assert len(ac.servers()) == 7
    assert len(aobj) == 7
    assert len(list(aobj.find(tag="normal"))) == 3
    for s in aobj.find(tag="normal"):
        assert s.priority == DapnetPriority.NORMAL

    assert len(list(aobj.find(tag="emerg"))) == 3
    for s in aobj.find(tag="emerg"):
        assert s.priority == DapnetPriority.EMERGENCY

    assert len(list(aobj.find(tag="dapnet_str"))) == 2
    assert len(list(aobj.find(tag="dapnet_str_int"))) == 2
    assert len(list(aobj.find(tag="dapnet_int"))) == 2

    assert len(list(aobj.find(tag="dapnet_invalid"))) == 1
    assert (
        next(aobj.find(tag="dapnet_invalid")).priority == DapnetPriority.NORMAL
    )

    # Notifications work
    assert aobj.notify(title="title", body="body") is True



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging

from helpers import AppriseURLTester

from apprise import Apprise
from apprise.plugins import jellyfin

logging.disable(logging.CRITICAL)

# Our Testing URLs
# Note: The majority of tests are already handled by the emby:// adaptation

apprise_url_tests = (
    # Insecure Request; no hostname specified
    (
        "jellyfin://",
        {
            "instance": None,
        },
    ),
    # Secure Request; no hostname specified
    (
        "jellyfins://",
        {
            "instance": None,
        },
    ),
    # No user specified
    (
        "jellyfin://localhost",
        {
            # Missing a username
            "instance": TypeError,
        },
    ),
    (
        "jellyfin://:@/",
        {
            "instance": None,
        },
    ),
    # Valid Authentication (we do not validate credentials here)
    (
        "jellyfin://l2g@localhost",
        {
            "instance": jellyfin.NotifyJellyfin,
            # Authentication can't be validated through these unit tests
            "response": False,
        },
    ),
    (
        "jellyfins://l2g:password@localhost",
        {
            "instance": jellyfin.NotifyJellyfin,
            "response": False,
            "privacy_url": "jellyfins://l2g:****@localhost",
        },
    ),
)


def test_plugin_jellyfin_urls():
    """NotifyJellyfin() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


def test_plugin_jellyfin_instantiation():
    """NotifyJellyfin() instantiation tests."""

    obj = Apprise.instantiate("jellyfin://l2g:l2gpass@localhost")
    assert isinstance(obj, jellyfin.NotifyJellyfin)

    obj = Apprise.instantiate("jellyfins://l2g:l2gpass@localhost")
    assert isinstance(obj, jellyfin.NotifyJellyfin)
